# -*- coding: utf-8 -*-

# pyramid-helpers -- Helpers to develop Pyramid applications
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2011-2015 Cyril Lacoux, Easter-eggs
# @app_url@
#
# This file is part of pyramid-helpers.
#
# pyramid-helpers is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# pyramid-helpers is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.authorization import ACLAuthorizationPolicy
from pyramid.config import Configurator
from pyramid.session import SignedCookieSessionFactory

from pyramid_helpers.auth import authentication_callback
from pyramid_helpers.models import initialize_model
from pyramid_helpers.resources import ArticlesResource
from pyramid_helpers.resources import TestResource


def main(global_config, **settings):
    """
    This function returns a Pyramid WSGI application.
    """

    # Model setup
    initialize_model(settings)

    # Authentication setup
    authentication_secret = global_config.get('authentication.secret', 'the-secret-used-for-authentication')
    authentication_policy = AuthTktAuthenticationPolicy(
        authentication_secret,
        callback=authentication_callback,
        hashalg='sha512',
    )
    authorization_policy = ACLAuthorizationPolicy()

    # Session setup
    session_key = global_config.get('session.key', 'the-secret-session-key')
    session_factory = SignedCookieSessionFactory(session_key)

    # Global config
    config = Configurator(
        authentication_policy=authentication_policy,
        authorization_policy=authorization_policy,
        session_factory=session_factory,
        settings=settings,
    )

    # Auth setup
    config.include('pyramid_helpers.auth')

    # I18n setup
    config.include('pyramid_helpers.i18n')

    # Forms setup
    config.include('pyramid_helpers.forms')

    # Pagers setup
    config.include('pyramid_helpers.paginate')

    # Add CSV renderer
    config.add_renderer('csv', 'pyramid_helpers.renderers.csv_renderer_factory')

    # Replace JSON renderer (callback support)
    config.add_renderer('json', 'pyramid_helpers.renderers.json_renderer_factory')

    # Static route
    config.add_static_view('static', 'pyramid_helpers:static')

    # Routes defined in auth.py
    config.add_route('login', '/login')
    config.add_route('login2', '/login2/', factory=TestResource)   # Basic authentication test #1
    config.add_route('login3', '/login3/', factory=TestResource)   # Basic authentication test #2
    config.add_route('logout', '/logout')

    # Scan auth and errors modules
    config.scan('pyramid_helpers.auth')
    config.scan('pyramid_helpers.errors')

    #
    # Applicative routes
    #

    # Routes defined in views/__init__.py
    config.add_route('index', '/')

    # Routes defined in views/articles.py
    config.add_route('articles.index', '/articles/', factory=ArticlesResource)
    config.add_route('articles.add', '/articles/add', factory=ArticlesResource)
    config.add_route('articles.view', '/articles/{article}', factory=ArticlesResource)
    config.add_route('articles.delete', '/articles/{article}/delete', factory=ArticlesResource)
    config.add_route('articles.edit', '/articles/{article}/edit', factory=ArticlesResource)
    config.add_route('articles.json', '/articles/{article}/json', factory=ArticlesResource)

    # Scan views modules
    config.scan('pyramid_helpers.views')

    return config.make_wsgi_app()
