# -*- coding: utf-8 -*-

# pyramid-helpers -- Helpers to develop Pyramid applications
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2011-2015 Cyril Lacoux, Easter-eggs
# @app_url@
#
# This file is part of pyramid-helpers.
#
# pyramid-helpers is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# pyramid-helpers is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import logging

from pyramid.httpexceptions import HTTPFound
from pyramid.security import forget
from pyramid.security import remember
from pyramid.view import view_config

from pyramid_helpers.forms import validate
from pyramid_helpers.forms.auth import LoginForm
from pyramid_helpers.models import DBSession
from pyramid_helpers.models import User


"""
Authentication module, usefull documentation can be found at
http://docs.pylonsproject.org/projects/pyramid/1.0/api/security.html
http://docs.pylonsproject.org/projects/pyramid_cookbook/dev/authentication.html
"""


log = logging.getLogger(__name__)


def get_user(username):
    if username is None:
        return None

    try:
        user = DBSession.query(User).filter_by(username=username).first()
    except:
        log.exception('Failed to get user with username={0}'.format(username))
        user = None

    return user

def on_before_renderer(event):
    """ Add authenticated_user and has_permission() to renderer context """
    request = event['request']
    event['authenticated_user'] = request.authenticated_user
    event['has_permission'] = request.has_permission


def on_new_request(event):
    """ Add authenticated_user to request """

    request = event.request
    request.authenticated_user = get_user(request.authenticated_userid)


def authentication_callback(userid, request):
    """ Called by authentication policy """

    user = get_user(userid)
    if user is None:
        # Invalid user
        return None

    return [
        ('group:%s' % group.name)
        for group in user.groups
    ]


def check_credentials(username, password):
    user = get_user(username)
    return user is not None and user.validate_password(password)


@view_config(route_name='login', renderer='/login.mako')
@validate('login_form', LoginForm)
def login(request):
    _ = request.translate
    form = request.forms['login_form']
    if request.method == 'POST':
        if not form.errors:
            username = form.result['username']
            password = form.result['password']
            came_from = form.result['came_from']

            if check_credentials(username, password):
                headers = remember(request, username)
                return HTTPFound(location=came_from, headers=headers)
            else:
                form.errors['username'] = _(u'Bad user or password')
    else:
        # Default data
        data = dict(came_from=request.referer if request.referer and request.referer != request.url else '/')
        form.set_data(data)

    return dict(
        title=_(u'Authentication'),
    )


@view_config(route_name='logout')
def logout(request):
    request.session.invalidate()
    headers = forget(request)
    return HTTPFound(location='/', headers=headers)


def includeme(config):
    """
    Set up standard configurator registrations. Use via:

    .. code-block:: python

    config = Configurator()
    config.include('pyramid_helpers.auth')
    """

    config.add_subscriber(on_before_renderer, 'pyramid.events.BeforeRender')
    config.add_subscriber(on_new_request, 'pyramid.events.NewRequest')
