# -*- coding: utf-8 -*-

# pyramid-helpers -- Helpers to develop Pyramid applications
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2011-2015 Cyril Lacoux, Easter-eggs
# @app_url@
#
# This file is part of pyramid-helpers.
#
# pyramid-helpers is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# pyramid-helpers is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" I18n helpers for Pyramid """

from babel.dates import format_date as format_date_
from babel.dates import format_time as format_time_
from formencode.api import set_stdtranslation


N_ = lambda x: x


def add_renderer_globals(event):
    """ Add _() function to renderer context """
    request = event['request']
    event['_'] = request.translate
    event['ungettext'] = request.pluralize
    event['localizer'] = request.localizer
    event['format_date'] = request.format_date
    event['format_datetime'] = request.format_datetime
    event['format_time'] = request.format_time


def add_localizer(event):
    """ Add localizer to request """
    request = event.request
    registry = request.registry
    settings = registry.settings or {}

    domain = settings.get('i18n.domain')
    localizer = request.localizer
    set_stdtranslation(languages=[localizer.locale_name, ])

    # Tanslations
    def auto_translate(string, domain=domain):
        return localizer.translate(string, domain=domain)

    def auto_pluralize(singular, plural, n, domain=domain):
        return localizer.pluralize(singular, plural, n, domain=domain)

    request.translate = auto_translate
    request.pluralize = auto_pluralize

    # Dates
    def format_date(dt, *args, **kwargs):
        return format_date_(dt, *args, locale=localizer.locale_name, **kwargs)

    def format_datetime(dt, *args, **kwargs):
        return u'%s %s' % (
            format_date(dt, *args, format='long', **kwargs),
            format_time(dt, *args, format='short', **kwargs),
        )

    def format_time(dt, *args, **kwargs):
        return format_time_(dt, *args, locale=localizer.locale_name, **kwargs)

    request.format_date = format_date
    request.format_datetime = format_datetime
    request.format_time = format_time


def header_locale_negotiator(request):
    """ Locale negotiation from Accept-Language header """
    registry = request.registry
    settings = registry.settings or {}
    available_languages = settings.get('i18n.available_languages', '').split()
    default_locale_name = settings.get('i18n.default_locale_name', 'en')

    locale_name = None
    if request.accept_language:
        locale_name = request.accept_language.best_match(available_languages)

    if locale_name is None:
        locale_name = default_locale_name

    return locale_name


def includeme(config):
    """
    Set up standard configurator registrations. Use via:

    .. code-block:: python

    config = Configurator()
    config.include('pyramid_helpers.i18n')
    """

    config.set_locale_negotiator(header_locale_negotiator)
    config.add_translation_dirs('/usr/share/locale/')
    config.add_subscriber(add_renderer_globals, 'pyramid.events.BeforeRender')
    config.add_subscriber(add_localizer, 'pyramid.events.NewRequest')
