# -*- coding: utf-8 -*-

# pyramid-helpers -- Helpers to develop Pyramid applications
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2011-2015 Cyril Lacoux, Easter-eggs
# @app_url@
#
# This file is part of pyramid-helpers.
#
# pyramid-helpers is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# pyramid-helpers is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import hashlib
import random
import string

from sqlalchemy import engine_from_config

from sqlalchemy.ext.declarative import declarative_base

from sqlalchemy.orm import relation
from sqlalchemy.orm import scoped_session
from sqlalchemy.orm import sessionmaker

from sqlalchemy.schema import Column
from sqlalchemy.schema import Table
from sqlalchemy.schema import ForeignKey

from sqlalchemy.types import DateTime
from sqlalchemy.types import Enum
from sqlalchemy.types import Integer
from sqlalchemy.types import Text
from sqlalchemy.types import Unicode

from zope.sqlalchemy import ZopeTransactionExtension

DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))
Base = declarative_base()

RANDOM_STRING = string.letters + string.digits


def random_string(self, length=10):
    return u''.join(
        random.choice(RANDOM_STRING)
        for i in range(0, length)
    )


# Users <-> Groups
user_groups_associations = Table(
    'user_groups_associations',
    Base.metadata,
    Column('user_id', Integer, ForeignKey('users.id')),
    Column('group_id', Integer, ForeignKey('groups.id')),
)


class Group(Base):
    __tablename__ = 'groups'

    # Primary key
    id = Column(Integer, primary_key=True)

    # Attributes
    name = Column(Unicode(255), unique=True)
    description = Column(Text)

    def to_json(self):
        return dict(
            name=self.name,
            description=self.description,
        )


class User(Base):
    __tablename__ = 'users'

    # Primary key
    id = Column(Integer, primary_key=True)

    # Attributes
    username = Column(Unicode(255), unique=True, nullable=False)
    password = Column(Unicode(32))
    firstname = Column(Unicode(255))
    lastname = Column(Unicode(255))

    # Relations
    groups = relation('Group', secondary=user_groups_associations)

    def __get_salted_hash(self, password, salt=None):
        if salt is None:
            salt = random_string(10)
        sha256 = hashlib.sha256(password)
        sha256.update(salt)
        return '%s:%s' % (salt, sha256.hexdigest())

    @property
    def fullname(self):
        fullname = []
        if self.firstname:
            fullname.append(self.firstname)
        if self.lastname:
            fullname.append(self.lastname)
        if not fullname:
            fullname.append(self.username)
        return u' '.join(fullname)

    def to_json(self):
        return dict(
            username=self.username,
            firstname=self.firstname,
            lastname=self.lastname,
            groups=[group.to_json() for group in self.groups]
        )

    def set_password(self, password):
        self.password = self.__get_salted_hash(password)

    def validate_password(self, password):
        if self.password is None:
            return False

        salt, salted = self.password.split(':')
        return self.password == self.__get_salted_hash(password, salt=salt)


class Article(Base):
    __tablename__ = 'articles'

    # Primary key
    id = Column(Integer, primary_key=True)

    # Foreign key
    author_id = Column(Integer, ForeignKey('users.id'))

    # Attributes
    creation_date = Column(DateTime, nullable=False)
    modification_date = Column(DateTime, nullable=False)
    title = Column(Unicode(255), unique=True, nullable=False)
    text = Column(Text)
    status = Column(Enum(u'draft', u'published', u'refused'))

    # Relations
    author = relation('User', backref='articles')

    def to_json(self):
        return dict(
            id=self.id,
            author=self.author.to_json(),
            creation_date=self.creation_date.isoformat(),
            title=self.title,
            text=self.text,
            status=self.status,
        )


def initialize_model(settings):
    engine = engine_from_config(settings, 'sqlalchemy.')
    DBSession.configure(bind=engine)
    Base.metadata.bind = engine
