# -*- coding: utf-8 -*-

# pyramid-helpers -- Helpers to develop Pyramid applications
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2011-2015 Cyril Lacoux, Easter-eggs
# @app_url@
#
# This file is part of pyramid-helpers.
#
# pyramid-helpers is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# pyramid-helpers is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from decorator import decorator


class Pager(list):
    def __init__(self, request, name, limit=10, sort=None, order=None, volatile_items=('format', ), **url_kwargs):

        # Calling inherited
        super(Pager, self).__init__()

        self.name = name
        self.request = request

        # Keys
        self.limit_key = '{0}.limit'.format(name)
        self.order_key = '{0}.order'.format(name)
        self.page_key = '{0}.page'.format(name)
        self.partial_key = '{0}.partial'.format(name)
        self.sort_key = '{0}.sort'.format(name)
        self.volatile_items = volatile_items + (self.page_key, self.partial_key)

        # Partial parameter
        self.partial = True if self.partial_key in request.params else False

        # Page parameter
        self.page = request.params.get(self.page_key)
        try:
            self.page = int(self.page)
        except:
            self.page = request.session.get(self.page_key, 1)

        # Limit parameter
        self.limit = request.params.get(self.limit_key)
        try:
            self.limit = int(self.limit)
        except:
            self.limit = request.session.get(self.limit_key, limit)

        self.sort = request.params.get(self.sort_key, sort)
        self.order = request.params.get(self.order_key, order)

        self.params = dict(
            (k, v)
            for k, v in request.params.mixed().items()
            if k not in self.volatile_items
        )

        # Empty list
        self.page_count = 0
        self.first_page = None
        self.last_page = None

        self.first_item = None
        self.last_item = None

        self.url_kwargs = url_kwargs

    def header_class(self, key):
        if self.sort == key:
            extra = 'glyphicon-sort-by-attributes'
            if self.order == 'desc':
                extra = '{0}-alt'.format(extra)
        else:
            extra = 'glyphicon-sort'
        return 'glyphicon {0}'.format(extra)

    def link(self, page=1, limit=None, sort=None, order=None):
        params = self.params.copy()
        if page == -1:
            page = self.last_page
        params[self.page_key] = page
        if limit:
            params[self.limit_key] = limit
        if sort:
            params[self.sort_key] = sort
        if order:
            if order == 'toggle':
                if self.sort == sort:
                    order = 'asc' if self.order == 'desc' else 'desc'
                else:
                    order = 'asc'
            params[self.order_key] = order
        return self.request.current_route_path(_query=params, **self.url_kwargs)

    def links(self, before=3, after=3):
        """ Compute pagination links """
        links = []
        for page in range(self.page - before, self.page + after + 1):
            if page < self.first_page or page > self.last_page:
                continue
            links.append((page, self.link(page=page)))
        return links

    def set_collection(self, collection):
        # Clear
        del self[:]

        self.item_count = collection.count()

        if self.item_count:
            self.page_count = ((self.item_count - 1) / self.limit) + 1
            self.first_page = 1
            self.last_page = self.page_count

            if self.page < self.first_page:
                self.page = self.first_page
            elif self.page > self.last_page:
                self.page = self.last_page

            self.first_item = (self.page - 1) * self.limit + 1
            self.last_item = min(self.first_item + self.limit - 1, self.item_count)

            items = collection[self.first_item - 1:self.last_item]
            self.extend(items)

        self.request.session[self.page_key] = self.page
        self.request.session[self.limit_key] = self.limit


def on_before_renderer(event):
    """ Add pagers and pager_ctx dictionnaries to renderer context """
    request = event['request']
    event['pagers'] = request.pagers
    event['pager_ctx'] = request.pager_ctx


def on_new_request(event):
    """
    Add pagers and pager_ctx dictionnaries to request

    pagers dictionnary will carry all pagers object using pager's name as key
    pager_ctx dictionnary will carry current pager context:
        'current' key is reserved for current pager object
    """
    request = event.request
    request.pagers = {}
    request.pager_ctx = {}


def paginate(name, limit=10, sort=None, order=None, partial_template=None, volatile_items=('format', )):

    def wrapper(func, request, *args, **kwargs):

        # Initialize pager object
        pager = Pager(request, name, limit=limit, sort=sort, order=order, volatile_items=volatile_items)

        # Store the pager
        request.pagers[name] = pager

        # Call function
        data = func(request, *args, **kwargs)
        if request.is_response(data):
            # View returned a response object
            return data

        if pager.partial and partial_template:
            # Override template if needed
            request.override_renderer = partial_template

        return data

    return decorator(wrapper)


def includeme(config):
    """
    Set up standard configurator registrations. Use via:

    .. code-block:: python

    config = Configurator()
    config.include('pyramid_helpers.paginate')
    """

    config.add_subscriber(on_before_renderer, 'pyramid.events.BeforeRender')
    config.add_subscriber(on_new_request, 'pyramid.events.NewRequest')
