# -*- coding: utf-8 -*-

# pyramid-helpers -- Helpers to develop Pyramid applications
# By: Cyril Lacoux <clacoux@easter-eggs.com>
#
# Copyright (C) 2011-2015 Cyril Lacoux, Easter-eggs
# @app_url@
#
# This file is part of pyramid-helpers.
#
# pyramid-helpers is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# pyramid-helpers is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime

from pyramid.httpexceptions import HTTPFound
from pyramid.httpexceptions import HTTPNotFound
from pyramid.view import view_config

from pyramid_helpers.forms import validate
from pyramid_helpers.forms.articles import ArticleForm
from pyramid_helpers.models import Article
from pyramid_helpers.models import DBSession
from pyramid_helpers.paginate import paginate


# Common functions
def get_article(request):
    article_id = request.matchdict.get('article')
    if article_id is None:
        return None
    return DBSession.query(Article).get(article_id)


# Views
@validate('article_form', ArticleForm)
def _edit(request, article=None):
    _ = request.translate
    form = request.forms['article_form']

    if request.method == 'POST':
        if not form.errors:
            same_title = DBSession.query(Article).filter_by(title=form.result['title']).first()
            if same_title and same_title != article:
                form.errors['title'] = _(u'Title already used in another article')

        if not form.errors:
            utcnow = datetime.datetime.utcnow()
            if article is None:
                article = Article()
                article.author = request.authenticated_user
                article.creation_date = utcnow

            article.title = form.result['title']
            article.text = form.result['text']
            article.status = form.result['status']

            if DBSession.is_modified(article):
                article.modification_date = utcnow

            # Flushing the session to get new id
            DBSession.flush()

            return HTTPFound(location=request.route_url('articles.view', article=article.id))

    elif article is not None:
        # Load values from article
        data = dict(
            title=article.title,
            text=article.text,
            status=article.status
        )
        form.set_data(data)

    else:
        # Default values
        data = dict()
        form.set_data(data)

    if article is None:
        breadcrumb = [
            (_(u'Home'), request.route_path('index')),
            (_(u'New article'), request.route_path('articles.add'))
        ]
        title = _(u'New article')
        cancel_link = request.route_path('articles.index')

    else:
        breadcrumb = [
            (_(u'Home'), request.route_path('index')),
            (article.title, request.route_path('articles.view', article=article.id)),
            (_(u'Edition'), request.route_path('articles.edit', article=article.id)),
        ]
        title = _(u'Article "%s" edition') % article.title
        cancel_link = request.route_url('articles.view', article=article.id)

    return dict(
        breadcrumb=breadcrumb,
        cancel_link=cancel_link,
        title=title,
    )


@view_config(permission='add', route_name='articles.add', renderer='/articles/edit.mako')
def add(request):
    return _edit(request)


@view_config(permission='delete', route_name='articles.delete', renderer='/confirm.mako')
def delete(request):
    _ = request.translate

    article = get_article(request)
    if article is None:
        raise HTTPNotFound(explanation=_(u'Invalid article'))

    if request.method == 'POST':
        if u'cancel' in request.params:
            return HTTPFound(location=request.route_url('articles.view', article=article.id))

        DBSession.delete(article)
        return HTTPFound(location=request.route_url('articles.index'))

    return dict(
        note=None,
        question=_(u'Do you really want to delete article "%s" ?') % article.title,
        title=_(u'Article "%s" deletion') % article.title
    )


@view_config(permission='edit', route_name='articles.edit', renderer='/articles/edit.mako')
def edit(request):
    _ = request.translate

    article = get_article(request)
    if article is None:
        raise HTTPNotFound(explanation=_(u'Invalid article'))

    return _edit(request, article=article)


@view_config(permission='view', route_name='articles.index', renderer='/articles/index.mako')
@paginate('articles', limit=10, sort='id', order='desc', partial_template='/articles/list.mako')
def index(request):
    _ = request.translate
    pager = request.pagers['articles']

    query = DBSession.query(Article)
    if not request.has_permission('edit'):
        query = query.filter_by(status=u'published')

    pager.set_collection(query)

    if pager.partial:
        return dict()

    if 'csv' in request.params:
        # CSV export
        request.override_renderer = 'csv'
        return dict(
            filename=_('articles.csv'),
            format=u'{id};"{title}";{creation_date};{status};"{text}"',
            header=_('id;title;creation date;status;text'),
            values=[article.to_json() for article in query],                # Using query instead of pager to get all results
        )

    breadcrumb = [
        (_(u'Home'), request.route_path('articles.index')),
    ]
    return dict(
        breadcrumb=breadcrumb,
        title=_(u'Articles')
    )


@view_config(permission='view', route_name='articles.json', renderer='json')
def json(request):
    return view(request)['article'].to_json()


@view_config(permission='view', route_name='articles.view', renderer='/articles/view.mako')
def view(request):
    _ = request.translate

    article = get_article(request)
    if article is None:
        raise HTTPNotFound(explanation=_(u'Invalid article'))

    breadcrumb = [
        (_(u'Home'), request.route_path('index')),
        (article.title, request.route_path('articles.view', article=article.id)),
    ]
    return dict(
        article=article,
        breadcrumb=breadcrumb,
        title=article.title
    )
